@@ -1,8 +1,8 @@
 import contextlib
-import functools
 import weakref
+import warnings
 
-from pytestqt.exceptions import SignalTimeoutError, TimeoutError
+from pytestqt.exceptions import TimeoutError
 from pytestqt.qt_compat import qt_api
 from pytestqt.wait_signal import (
     SignalBlocker,
@@ -23,7 +23,7 @@ def _parse_ini_boolean(value):
         raise ValueError("unknown string for bool: %r" % value)
 
 
-class QtBot(object):
+class QtBot:
     """
     Instances of this class are responsible for sending events to `Qt` objects (usually widgets),
     simulating user input.
@@ -38,7 +38,7 @@ class QtBot(object):
     .. automethod:: waitActive
     .. automethod:: waitExposed
     .. automethod:: waitForWindowShown
-    .. automethod:: stopForInteraction
+    .. automethod:: stop
     .. automethod:: wait
 
     **Signals and Events**
@@ -59,7 +59,7 @@ class QtBot(object):
     Below are methods used to simulate sending key events to widgets:
 
     .. staticmethod:: keyClick (widget, key[, modifier=Qt.NoModifier[, delay=-1]])
-    .. staticmethod:: keyClicks (widget, key sequence[, modifier=Qt.NoModifier[, delay=-1]])
+    .. staticmethod:: keyClicks (widget, key_sequence[, modifier=Qt.NoModifier[, delay=-1]])
     .. staticmethod:: keyEvent (action, widget, key[, modifier=Qt.NoModifier[, delay=-1]])
     .. staticmethod:: keyPress (widget, key[, modifier=Qt.NoModifier[, delay=-1]])
     .. staticmethod:: keyRelease (widget, key[, modifier=Qt.NoModifier[, delay=-1]])
@@ -83,12 +83,12 @@ class QtBot(object):
             * ``Qt.KeypadModifier``: A keypad button is pressed.
             * ``Qt.GroupSwitchModifier``: X11 only. A Mode_switch key on the keyboard is pressed.
 
-        :param int delay: after the event, delay the test for this miliseconds (if > 0).
+        :param int delay: after the event, delay the test for this milliseconds (if > 0).
 
 
     .. staticmethod:: keyToAscii (key)
 
-        Auxilliary method that converts the given constant ot its equivalent ascii.
+        Auxiliary method that converts the given constant ot its equivalent ascii.
 
         :param Qt.Key_* key: one of the constants for keys in the Qt namespace.
 
@@ -114,22 +114,22 @@ class QtBot(object):
         :param Qt.MouseButton button: flags OR'ed together representing the button pressed.
             Possible flags are:
 
-            * ``Qt.NoButton``: The button state does not refer to any button
+            * ``Qt.MouseButton.NoButton``: The button state does not refer to any button
               (see QMouseEvent.button()).
-            * ``Qt.LeftButton``: The left button is pressed, or an event refers to the left button.
+            * ``Qt.MouseButton.LeftButton``: The left button is pressed, or an event refers to the left button.
               (The left button may be the right button on left-handed mice.)
-            * ``Qt.RightButton``: The right button.
-            * ``Qt.MidButton``: The middle button.
-            * ``Qt.MiddleButton``: The middle button.
-            * ``Qt.XButton1``: The first X button.
-            * ``Qt.XButton2``: The second X button.
+            * ``Qt.MouseButton.RightButton``: The right button.
+            * ``Qt.MouseButton.MidButton``: The middle button.
+            * ``Qt.MouseButton.MiddleButton``: The middle button.
+            * ``Qt.MouseButton.XButton1``: The first X button.
+            * ``Qt.MouseButton.XButton2``: The second X button.
 
         :param Qt.KeyboardModifier modifier: flags OR'ed together representing other modifier keys
             also pressed. See `keyboard modifiers`_.
 
         :param QPoint position: position of the mouse pointer.
 
-        :param int delay: after the event, delay the test for this miliseconds (if > 0).
+        :param int delay: after the event, delay the test for this milliseconds (if > 0).
 
 
     .. _QTest API: http://doc.qt.io/qt-5/qtest.html
@@ -141,18 +141,15 @@ class QtBot(object):
 
     def _should_raise(self, raising_arg):
         ini_val = self._request.config.getini("qt_default_raising")
-        legacy_ini_val = self._request.config.getini("qt_wait_signal_raising")
 
         if raising_arg is not None:
             return raising_arg
-        elif legacy_ini_val:
-            return _parse_ini_boolean(legacy_ini_val)
         elif ini_val:
             return _parse_ini_boolean(ini_val)
         else:
             return True
 
-    def addWidget(self, widget, **kwargs):
+    def addWidget(self, widget, *, before_close_func=None):
         """
         Adds a widget to be tracked by this bot. This is not required, but will ensure that the
         widget gets closed by the end of the test, so it is highly recommended.
@@ -166,11 +163,11 @@ class QtBot(object):
 
         .. note:: This method is also available as ``add_widget`` (pep-8 alias)
         """
-        _add_widget(self._request.node, widget, **kwargs)
+        if not isinstance(widget, qt_api.QtWidgets.QWidget):
+            raise TypeError("Need to pass a QWidget to addWidget!")
+        _add_widget(self._request.node, widget, before_close_func=before_close_func)
 
-    add_widget = addWidget  # pep-8 alias
-
-    def waitActive(self, widget, timeout=1000):
+    def waitActive(self, widget, *, timeout=5000):
         """
         Context manager that waits for ``timeout`` milliseconds or until the window is active.
         If window is not exposed within ``timeout`` milliseconds, raise ``TimeoutError``.
@@ -189,10 +186,6 @@ class QtBot(object):
         :param int|None timeout:
             How many milliseconds to wait for.
 
-        .. note::
-            This function is only available in PyQt5, raising a ``RuntimeError`` if called from
-            ``PyQt4`` or ``PySide``.
-
         .. note:: This method is also available as ``wait_active`` (pep-8 alias)
         """
         __tracebackhide__ = True
@@ -200,9 +193,7 @@ class QtBot(object):
             "qWaitForWindowActive", "activated", widget, timeout
         )
 
-    wait_active = waitActive  # pep-8 alias
-
-    def waitExposed(self, widget, timeout=1000):
+    def waitExposed(self, widget, *, timeout=5000):
         """
         Context manager that waits for ``timeout`` milliseconds or until the window is exposed.
         If the window is not exposed within ``timeout`` milliseconds, raise ``TimeoutError``.
@@ -221,10 +212,6 @@ class QtBot(object):
         :param int|None timeout:
             How many milliseconds to wait for.
 
-        .. note::
-            This function is only available in PyQt5, raising a ``RuntimeError`` if called from
-            ``PyQt4`` or ``PySide``.
-
         .. note:: This method is also available as ``wait_exposed`` (pep-8 alias)
         """
         __tracebackhide__ = True
@@ -232,29 +219,38 @@ class QtBot(object):
             "qWaitForWindowExposed", "exposed", widget, timeout
         )
 
-    wait_exposed = waitExposed  # pep-8 alias
-
     def waitForWindowShown(self, widget):
         """
         Waits until the window is shown in the screen. This is mainly useful for asynchronous
         systems like X11, where a window will be mapped to screen some time after being asked to
         show itself on the screen.
 
+        .. warning::
+            This method does **not** raise ``TimeoutError`` if the window wasn't shown.
+
+        .. deprecated:: 4.0
+            Use the qtbot.waitForWindowExposed context manager instead.
+
         :param QWidget widget:
             Widget to wait on.
 
-        .. note:: In ``PyQt5`` this function is considered deprecated in favor of :meth:`waitExposed`.
+        :returns:
+            ``True`` if the window was shown, ``False`` if ``.show()`` was never
+            called or a timeout occurred.
 
         .. note:: This method is also available as ``wait_for_window_shown`` (pep-8 alias)
         """
-        if hasattr(qt_api.QtTest.QTest, "qWaitForWindowExposed"):
-            return qt_api.QtTest.QTest.qWaitForWindowExposed(widget)
-        else:
-            return qt_api.QtTest.QTest.qWaitForWindowShown(widget)
+        warnings.warn(
+            "waitForWindowShown is deprecated, as the underlying Qt method was "
+            "obsoleted in Qt 5.0 and removed in Qt 6.0. Its name is imprecise and "
+            "the pytest-qt wrapper does not raise TimeoutError if the window "
+            "wasn't shown. Please use the qtbot.waitExposed context manager "
+            "instead.",
+            DeprecationWarning,
+        )
+        return qt_api.QtTest.QTest.qWaitForWindowExposed(widget)
 
-    wait_for_window_shown = waitForWindowShown  # pep-8 alias
-
-    def stopForInteraction(self):
+    def stop(self):
         """
         Stops the current test flow, letting the user interact with any visible widget.
 
@@ -263,8 +259,6 @@ class QtBot(object):
 
         Closing the windows should resume the test run, with ``qtbot`` attempting to restore visibility
         of the widgets as they were before this call.
-
-        .. note:: As a convenience, it is also aliased as `stop`.
         """
         widget_and_visibility = []
         for weak_widget in _iter_widgets(self._request.node):
@@ -272,14 +266,12 @@ class QtBot(object):
             if widget is not None:
                 widget_and_visibility.append((widget, widget.isVisible()))
 
-        qt_api.QApplication.instance().exec_()
+        qt_api.exec(qt_api.QtWidgets.QApplication.instance())
 
         for widget, visible in widget_and_visibility:
             widget.setVisible(visible)
 
-    stop = stopForInteraction
-
-    def waitSignal(self, signal=None, timeout=1000, raising=None, check_params_cb=None):
+    def waitSignal(self, signal, *, timeout=5000, raising=None, check_params_cb=None):
         """
         .. versionadded:: 1.2
 
@@ -311,7 +303,7 @@ class QtBot(object):
 
         :param Signal signal:
             A signal to wait for, or a tuple ``(signal, signal_name_as_str)`` to improve the error message that is part
-            of ``TimeoutError``. Set to ``None`` to just use timeout.
+            of ``TimeoutError``.
         :param int timeout:
             How many milliseconds to wait before resuming control flow.
         :param bool raising:
@@ -326,27 +318,25 @@ class QtBot(object):
         :returns:
             ``SignalBlocker`` object. Call ``SignalBlocker.wait()`` to wait.
 
-        .. note::
-            Cannot have both ``signals`` and ``timeout`` equal ``None``, or
-            else you will block indefinitely. We throw an error if this occurs.
-
         .. note::
             This method is also available as ``wait_signal`` (pep-8 alias)
         """
+        if signal is None:
+            raise ValueError(
+                f"Passing None as signal isn't supported anymore, use qtbot.wait({timeout}) instead."
+            )
         raising = self._should_raise(raising)
         blocker = SignalBlocker(
             timeout=timeout, raising=raising, check_params_cb=check_params_cb
         )
-        if signal is not None:
-            blocker.connect(signal)
+        blocker.connect(signal)
         return blocker
 
-    wait_signal = waitSignal  # pep-8 alias
-
     def waitSignals(
         self,
-        signals=None,
-        timeout=1000,
+        signals,
+        *,
+        timeout=5000,
         raising=None,
         check_params_cbs=None,
         order="none",
@@ -375,7 +365,6 @@ class QtBot(object):
         :param list signals:
             A list of :class:`Signal` objects to wait for. Alternatively: a list of (``Signal, str``) tuples of the form
             ``(signal, signal_name_as_str)`` to improve the error message that is part of ``TimeoutError``.
-            Set to ``None`` to just use timeout.
         :param int timeout:
             How many milliseconds to wait before resuming control flow.
         :param bool raising:
@@ -404,15 +393,16 @@ class QtBot(object):
             ``MultiSignalBlocker`` object. Call ``MultiSignalBlocker.wait()``
             to wait.
 
-        .. note::
-           Cannot have both ``signals`` and ``timeout`` equal ``None``, or
-           else you will block indefinitely. We throw an error if this occurs.
-
         .. note:: This method is also available as ``wait_signals`` (pep-8 alias)
         """
         if order not in ["none", "simple", "strict"]:
             raise ValueError("order has to be set to 'none', 'simple' or 'strict'")
 
+        if not signals:
+            raise ValueError(
+                f"Passing {signals} as signals isn't supported anymore, consider using qtbot.wait({timeout}) instead."
+            )
+
         raising = self._should_raise(raising)
 
         if check_params_cbs:
@@ -429,12 +419,9 @@ class QtBot(object):
             order=order,
             check_params_cbs=check_params_cbs,
         )
-        if signals is not None:
-            blocker.add_signals(signals)
+        blocker.add_signals(signals)
         return blocker
 
-    wait_signals = waitSignals  # pep-8 alias
-
     def wait(self, ms):
         """
         .. versionadded:: 1.9
@@ -448,7 +435,7 @@ class QtBot(object):
         blocker.wait()
 
     @contextlib.contextmanager
-    def assertNotEmitted(self, signal, wait=0):
+    def assertNotEmitted(self, signal, *, wait=0):
         """
         .. versionadded:: 1.11
 
@@ -469,9 +456,7 @@ class QtBot(object):
             yield
         spy.assert_not_emitted()
 
-    assert_not_emitted = assertNotEmitted  # pep-8 alias
-
-    def waitUntil(self, callback, timeout=1000):
+    def waitUntil(self, callback, *, timeout=5000):
         """
         .. versionadded:: 2.0
 
@@ -485,6 +470,8 @@ class QtBot(object):
 
             def view_updated():
                 assert view_model.count() > 10
+
+
             qtbot.waitUntil(view_updated)
 
         Another possibility is for ``callback()`` to return ``True`` when the desired condition
@@ -516,12 +503,14 @@ class QtBot(object):
             elapsed_ms = elapsed * 1000
             return elapsed_ms > timeout
 
+        timeout_msg = f"waitUntil timed out in {timeout} milliseconds"
+
         while True:
             try:
                 result = callback()
-            except AssertionError:
+            except AssertionError as e:
                 if timed_out():
-                    raise
+                    raise TimeoutError(timeout_msg) from e
             else:
                 if result not in (None, True, False):
                     msg = "waitUntil() callback must return None, True or False, returned %r"
@@ -534,15 +523,11 @@ class QtBot(object):
                 # 'True/False' form
                 if result:
                     return
-                else:
-                    assert not timed_out(), (
-                        "waitUntil timed out in %s miliseconds" % timeout
-                    )
+                if timed_out():
+                    raise TimeoutError(timeout_msg)
             self.wait(10)
 
-    wait_until = waitUntil  # pep-8 alias
-
-    def waitCallback(self, timeout=1000, raising=None):
+    def waitCallback(self, *, timeout=5000, raising=None):
         """
         .. versionadded:: 3.1
 
@@ -583,8 +568,6 @@ class QtBot(object):
         blocker = CallbackBlocker(timeout=timeout, raising=raising)
         return blocker
 
-    wait_callback = waitCallback  # pep-8 alias
-
     @contextlib.contextmanager
     def captureExceptions(self):
         """
@@ -610,59 +593,98 @@ class QtBot(object):
 
     capture_exceptions = captureExceptions
 
-    @classmethod
-    def _inject_qtest_methods(cls):
-        """
-        Injects QTest methods into the given class QtBot, so the user can access
-        them directly without having to import QTest.
-        """
+    @staticmethod
+    def keyClick(*args, **kwargs):
+        qt_api.QtTest.QTest.keyClick(*args, **kwargs)
 
-        def create_qtest_proxy_method(method_name):
+    @staticmethod
+    def keyClicks(*args, **kwargs):
+        qt_api.QtTest.QTest.keyClicks(*args, **kwargs)
 
-            if hasattr(qt_api.QtTest.QTest, method_name):
-                qtest_method = getattr(qt_api.QtTest.QTest, method_name)
+    @staticmethod
+    def keyEvent(*args, **kwargs):
+        qt_api.QtTest.QTest.keyEvent(*args, **kwargs)
 
-                def result(*args, **kwargs):
-                    return qtest_method(*args, **kwargs)
+    @staticmethod
+    def keyPress(*args, **kwargs):
+        qt_api.QtTest.QTest.keyPress(*args, **kwargs)
 
-                functools.update_wrapper(result, qtest_method)
-                return staticmethod(result)
-            else:
-                return None  # pragma: no cover
+    @staticmethod
+    def keyRelease(*args, **kwargs):
+        qt_api.QtTest.QTest.keyRelease(*args, **kwargs)
 
-        # inject methods from QTest into QtBot
-        method_names = [
-            "keyPress",
-            "keyClick",
-            "keyClicks",
-            "keyEvent",
-            "keyPress",
-            "keyRelease",
-            "keyToAscii",
-            "mouseClick",
-            "mouseDClick",
-            "mouseMove",
-            "mousePress",
-            "mouseRelease",
-        ]
-        for method_name in method_names:
-            method = create_qtest_proxy_method(method_name)
-            if method is not None:
-                setattr(cls, method_name, method)
+    @staticmethod
+    def keySequence(widget, key_sequence):
+        if not hasattr(qt_api.QtTest.QTest, "keySequence"):
+            raise NotImplementedError("This method is available from Qt 5.10 upwards.")
+        qt_api.QtTest.QTest.keySequence(widget, key_sequence)
+
+    @staticmethod
+    def keyToAscii(key):
+        if not hasattr(qt_api.QtTest.QTest, "keyToAscii"):
+            raise NotImplementedError("This method isn't available on PyQt5.")
+        qt_api.QtTest.QTest.keyToAscii(key)
+
+    @staticmethod
+    def mouseClick(*args, **kwargs):
+        qt_api.QtTest.QTest.mouseClick(*args, **kwargs)
+
+    @staticmethod
+    def mouseDClick(*args, **kwargs):
+        qt_api.QtTest.QTest.mouseDClick(*args, **kwargs)
+
+    @staticmethod
+    def mouseMove(*args, **kwargs):
+        qt_api.QtTest.QTest.mouseMove(*args, **kwargs)
+
+    @staticmethod
+    def mousePress(*args, **kwargs):
+        qt_api.QtTest.QTest.mousePress(*args, **kwargs)
+
+    @staticmethod
+    def mouseRelease(*args, **kwargs):
+        qt_api.QtTest.QTest.mouseRelease(*args, **kwargs)
+
+    # pep-8 aliases
+
+    def add_widget(self, *args, **kwargs):
+        return self.addWidget(*args, **kwargs)
+
+    def wait_active(self, *args, **kwargs):
+        return self.waitActive(*args, **kwargs)
+
+    def wait_exposed(self, *args, **kwargs):
+        return self.waitExposed(*args, **kwargs)
+
+    def wait_for_window_shown(self, *args, **kwargs):
+        return self.waitForWindowShown(*args, **kwargs)
+
+    def wait_signal(self, *args, **kwargs):
+        return self.waitSignal(*args, **kwargs)
+
+    def wait_signals(self, *args, **kwargs):
+        return self.waitSignals(*args, **kwargs)
+
+    def assert_not_emitted(self, *args, **kwargs):
+        return self.assertNotEmitted(*args, **kwargs)
+
+    def wait_until(self, *args, **kwargs):
+        return self.waitUntil(*args, **kwargs)
+
+    def wait_callback(self, *args, **kwargs):
+        return self.waitCallback(*args, **kwargs)
 
 
 # provide easy access to exceptions to qtbot fixtures
-QtBot.SignalTimeoutError = SignalTimeoutError
 QtBot.SignalEmittedError = SignalEmittedError
 QtBot.TimeoutError = TimeoutError
 QtBot.CallbackCalledTwiceError = CallbackCalledTwiceError
 
 
-def _add_widget(item, widget, **kwargs):
+def _add_widget(item, widget, *, before_close_func=None):
     """
     Register a widget into the given pytest item for later closing.
     """
-    before_close_func = kwargs.pop("before_close_func", None)
     qt_widgets = getattr(item, "qt_widgets", [])
     qt_widgets.append((weakref.ref(widget), before_close_func))
     item.qt_widgets = qt_widgets
@@ -692,7 +714,7 @@ def _iter_widgets(item):
     return (w for (w, _) in qt_widgets)
 
 
-class _WaitWidgetContextManager(object):
+class _WaitWidgetContextManager:
     """
     Context manager implementation used by ``waitActive`` and ``waitExposed`` methods.
     """
@@ -711,8 +733,6 @@ class _WaitWidgetContextManager(object):
 
     def __enter__(self):
         __tracebackhide__ = True
-        if qt_api.pytest_qt_api != "pyqt5":
-            raise RuntimeError("Available in PyQt5 only")
         return self
 
     def __exit__(self, exc_type, exc_val, exc_tb):
